/*
 *
 *  Copyright (C) 2014 Jürg Müller, CH-5524
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation version 3 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program. If not, see http://www.gnu.org/licenses/ .
 */

#if !defined(NCanUtils_H) 

  #define NCanUtils_H

  #define High(A)     (sizeof(A)/sizeof(A[0]) - 1)

  #include <stdio.h>

  struct KCanFrame
  {
    unsigned Counter;
    int      TimeStampDay;
    int      TimeStampMs;
    unsigned Id;
    int      Len;
    unsigned Flags;
    unsigned char Data[8];
    
    KCanFrame();
    void Init();
    void InitElsterFrame(unsigned short sender_id,
                         unsigned short receiver_id,
                         unsigned short elster_idx);

    bool EqualData(const KCanFrame & Frame) const;
    bool IsAnswer(const KCanFrame & Frame) const;
    bool Supported() const;
    unsigned Value() const;
    bool SetValue(unsigned short value);
    int GetElsterIdx() const;
    unsigned short ReceiverId() const;
    bool Changed(const KCanFrame & SendFrame) const;
    bool GetDataFromStream(const char * str);
    void FormMySqlInsert(char * statement) const;
    void FormHttpVariables(char * stream) const;
    // format: 302.0016
    bool SetElsterFrame(const char * str);
  };

  struct KComfortFrame
  {
    typedef enum
    {
      mt_init,
      mt_get,
      mt_change,
      mt_short_change,
      mt_invalid,
    } msg_type;

    typedef struct
    {
      unsigned recv_id;
      unsigned elster_idx;
    } t_scan_geraete_id;

    unsigned char Data[12];
    static unsigned short InitId;
    static const t_scan_geraete_id scan_geraete_id[];
    
    unsigned GetCheckSum() const;
    unsigned GetSum() const;
    bool CheckSum() const;
    void SetCheckSum();
    bool SetCanFrame(KCanFrame & Frame) const;
    bool GetCanFrame(const KCanFrame & Frame);
    void SetOk(bool Ok = true);
    void SetOkValue(unsigned Value);
    bool EqualData(const KComfortFrame & Frame) const;
    void SetToString(char * Str) const;
    bool SetFromString(const char * Str, bool AppendCheck = true);
    bool IsInitFrame() const;
    void SetInitOk();
    msg_type GetMsgType() const;
  };

  struct KCan232Frame
  {
    // T14611234 4 00010203             1 + 8 + 1 + 2*8 = 26 Byte max.
    // t550      8 aabbccddeeff0a0b
    char msg[32];
  
    KCan232Frame(const char * str = 0);
    bool SetCanFrame(KCanFrame & Frame) const;
    bool GetCanFrame(const KCanFrame & Frame);
    bool IsOk() const;
    KCan232Frame & operator =(const char * str);
  };

  class KStream
  {
    protected:
      unsigned char * mMemory;
      unsigned long mMemorySize;           // size of mMemory
      unsigned long mSize;                 // used size
      
    public:
      
      KStream();
      ~KStream();
      
      const unsigned char * GetMemory() const { return mMemory; };
      unsigned long Size() const { return mSize; };
      unsigned long MemorySize() const { return mMemorySize; };
    
      void AppendString(const char * str);
      void Append(const unsigned char * Stream, unsigned StreamLen);
      bool InsertBytes(unsigned long Offset, long Count);
      bool ReplaceBytes(const unsigned char * new_bytes, unsigned long Offset,
                        unsigned new_count, unsigned old_count = 0);
      
      // Wenn NewSize = 0: der Speicher mMemory wird freigegeben
      // Wenn NewSize < mMemorySize: mSize wird gleich NewSize gesetzt
      // sonst: es wird ein neuer Speicher alloziert. Vorhandene Daten
      //        werden in den neuen Speicher kopiert
      void SetSize(unsigned long NewSize);
      bool CheckSize(unsigned long NewSize);
      // mSize wird gleich Length gesetzt, aber höchstens gleich mMemorySize
      // (d.h. mMemory bleibt unverändert)
      void SetLength(unsigned long Length);
      
      bool ReadFile(const char * Filename);
      
  };


  namespace NCanUtils
  {
    void SleepMs(int aMs);
    void ArtTime(const struct timeval & Time, int & aDays, int & aMs);
    void ArtTime(int & aDays, int & aMs);
    bool GetArtDays(int Year, int Month, int Day, int & Days);
    void GetDateFromDays(int aDays, int & aDay, int & aMonth, int & aYear);
    void PrintTime(const KCanFrame & Frame, char * stream, bool csv);
    void PrintTime(int time_days, int time_ms, char * OutStr, bool csv);
    void PrintMs(int days, int ms, char * stream);
    void PrintDateTime(int days, int ms, char * stream);
    void FormatFrame(const KCanFrame & Frame, bool csv, char * OutStr);
    bool IsHexDigit(char aDigit);
    int GetHexDigit(char aDigit);
    unsigned GetHex(const char * & Line);
    bool GetHexByte(const char * & Line, unsigned char & byte);
    bool GetInt(const char * & Line, int & res);
    unsigned GetInt(int & res, const char * Line);
    
    long FileSize(FILE * InpFile);
    long FileSize(const char * Filename);
    bool ReadFileBuffer(FILE * InpFile, char * & Stream, long & FileLen);
    bool ReadFile(const char * Filename, char * & stream, long & FileLen);
  }
#if defined(__CAN__)
  extern unsigned gMainThreadId;
  extern unsigned gProcessId;
#endif

#endif

